
import { Suspense, lazy, memo, useEffect, useState } from "react";
import { Toaster } from "@/components/ui/toaster";
import { Toaster as Sonner } from "@/components/ui/sonner";
import { TooltipProvider } from "@/components/ui/tooltip";
import { QueryClient, QueryClientProvider } from "@tanstack/react-query";
import { BrowserRouter, Routes, Route, useLocation } from "react-router-dom";
import { Navigation } from "@/components/ui/navigation";
import { Footer } from "@/components/ui/footer";
import { WhatsAppButton } from "@/components/ui/whatsapp-button";

// Eagerly load critical route
import Index from "@/pages/Index";

// Lazily load non-critical routes with prefetching when idle
const NotFound = lazy(() => import("@/pages/NotFound"));
const Properties = lazy(() => import("@/pages/Properties"));
const PropertyDetail = lazy(() => import("@/pages/PropertyDetail"));
const Account = lazy(() => import("@/pages/Account"));
const Contact = lazy(() => import("@/pages/Contact"));
const Admin = lazy(() => import("@/pages/Admin"));

// Scroll to top component for route changes
const ScrollToTop = () => {
  const { pathname } = useLocation();

  useEffect(() => {
    window.scrollTo(0, 0);
  }, [pathname]);

  return null;
};

// Memoized loading component for better performance
const PageLoader = memo(() => (
  <div className="min-h-[70vh] w-full flex items-center justify-center">
    <div className="animate-spin h-12 w-12 border-t-2 border-b-2 border-primary"></div>
  </div>
));

PageLoader.displayName = 'PageLoader';

// Create QueryClient with optimized settings
const queryClient = new QueryClient({
  defaultOptions: {
    queries: {
      refetchOnWindowFocus: false,
      staleTime: 60 * 1000, // 1 minute
      gcTime: 10 * 60 * 1000, // 10 minutes
      retry: 1, // Only retry once to improve perceived performance
    },
  },
});

// Prefetch critical assets when browser is idle
if (typeof window !== 'undefined' && 'requestIdleCallback' in window) {
  window.requestIdleCallback(() => {
    // Preload key non-critical routes after initial render
    import("@/pages/Properties");
    import("@/pages/PropertyDetail");
  }, { timeout: 2000 });
}

// Route change handler component for analytics and performance monitoring
const RouteChangeHandler = () => {
  const location = useLocation();
  
  useEffect(() => {
    // Mark route change for performance measurement
    if ('performance' in window) {
      performance.mark('route-change-start');
      
      // Measure after animation frame to catch render time
      requestAnimationFrame(() => {
        requestAnimationFrame(() => {
          performance.mark('route-change-end');
          try {
            performance.measure('route-change', 'route-change-start', 'route-change-end');
            const entry = performance.getEntriesByName('route-change').pop();
            if (entry && process.env.NODE_ENV === 'development') {
              console.log(`Route change to ${location.pathname} took ${entry.duration.toFixed(1)}ms`);
            }
          } catch (e) {
            // Ignore measurement errors
          }
        });
      });
    }
    
    // Pre-cache linked routes
    setTimeout(() => {
      const links = document.querySelectorAll('a[href^="/"]');
      links.forEach(link => {
        const href = link.getAttribute('href');
        if (href && href !== location.pathname) {
          const route = href.split('/')[1];
          
          if (route === 'properties') {
            import("@/pages/Properties").catch(e => {});
          } else if (route === 'property') {
            import("@/pages/PropertyDetail").catch(e => {});
          } else if (route === 'contact') {
            import("@/pages/Contact").catch(e => {});
          }
        }
      });
    }, 1000);
  }, [location.pathname]);
  
  return null;
};

// Memoized route components to prevent unnecessary re-renders
const IndexRoute = memo(() => <Index />);
const NotFoundRoute = memo(() => <NotFound />);
const PropertiesRoute = memo(() => (
  <Suspense fallback={<PageLoader />}>
    <Properties />
  </Suspense>
));
const PropertyDetailRoute = memo(() => (
  <Suspense fallback={<PageLoader />}>
    <PropertyDetail />
  </Suspense>
));
const AccountRoute = memo(() => (
  <Suspense fallback={<PageLoader />}>
    <Account />
  </Suspense>
));
const ContactRoute = memo(() => (
  <Suspense fallback={<PageLoader />}>
    <Contact />
  </Suspense>
));
const AdminRoute = memo(() => (
  <Suspense fallback={<PageLoader />}>
    <Admin />
  </Suspense>
));

// Set display names for the memoized components
IndexRoute.displayName = 'IndexRoute';
NotFoundRoute.displayName = 'NotFoundRoute';
PropertiesRoute.displayName = 'PropertiesRoute';
PropertyDetailRoute.displayName = 'PropertyDetailRoute';
AccountRoute.displayName = 'AccountRoute';
ContactRoute.displayName = 'ContactRoute';
AdminRoute.displayName = 'AdminRoute';

// App Shell component for faster initial render
const AppShell = memo(() => {
  const [isReady, setIsReady] = useState(false);
  
  useEffect(() => {
    // Mark when app shell is hydrated
    performance.mark('app-shell-ready');
    
    // Give time for the UI to render before showing routes
    requestAnimationFrame(() => {
      setTimeout(() => {
        setIsReady(true);
        performance.mark('full-app-ready');
      }, 0);
    });
  }, []);
  
  return (
    <QueryClientProvider client={queryClient}>
      <TooltipProvider>
        <div className="overflow-x-hidden w-screen max-w-full">
          <Toaster />
          <Sonner />
          <BrowserRouter>
            <ScrollToTop />
            <RouteChangeHandler />
            <Navigation />
            <main className="min-h-screen w-full max-w-full overflow-x-hidden">
              {isReady && (
                <Routes>
                  <Route path="/" element={<IndexRoute />} />
                  <Route path="/properties" element={<PropertiesRoute />} />
                  <Route path="/property/:id" element={<PropertyDetailRoute />} />
                  <Route path="/account" element={<AccountRoute />} />
                  <Route path="/contact" element={<ContactRoute />} />
                  <Route path="/admin" element={<AdminRoute />} />
                  {/* ADD ALL CUSTOM ROUTES ABOVE THE CATCH-ALL "*" ROUTE */}
                  <Route path="*" element={<NotFoundRoute />} />
                </Routes>
              )}
            </main>
            <WhatsAppButton />
            <Footer />
          </BrowserRouter>
        </div>
      </TooltipProvider>
    </QueryClientProvider>
  );
});

AppShell.displayName = 'AppShell';

const App = () => <AppShell />;

export default App;
